﻿
using System;
using System.Collections.Generic;
namespace HeuristicLab.Problems.RoutePlanning.Graph {
  public class Graph {
    private IDictionary<long, Vertex> vertices;
    private IDictionary<long, List<Edge<Vertex>>> adjacencyMap;
    private IDictionary<long, List<Edge<Vertex>>> incidenceMap;

    public Graph() {
      vertices = new Dictionary<long, Vertex>();
      adjacencyMap = new Dictionary<long, List<Edge<Vertex>>>();
    }

    public void AddVertex(Vertex vertex) {
      vertices.Add(vertex.Id, vertex);
    }

    public Vertex GetVertex(long id) {
      Vertex vertex = null;
      vertices.TryGetValue(id, out vertex);
      return vertex;
    }

    public void AddEdge(long startId, long endId) {
      Vertex start = GetVertex(startId);
      if (start == null) throw new ArgumentException(string.Format("Invalid vertex id {0}", startId), "startId");
      Vertex end = GetVertex(startId);
      if (end == null) throw new ArgumentException(string.Format("Invalid vertex id {0}", endId), "endId");

      Edge<Vertex> edge = new Edge<Vertex>(start, end);
      //edge.Category = -1

      if (adjacencyMap[start.Id] == null) {
        adjacencyMap[start.Id] = new List<Edge<Vertex>>();
      }
      adjacencyMap[start.Id].Add(edge);

      if (incidenceMap[end.Id] == null) {
        incidenceMap[end.Id] = new List<Edge<Vertex>>();
      }
      incidenceMap[end.Id].Add(edge);
    }

    public Edge<Vertex> GetEdge(long startId, long endId) {
      List<Edge<Vertex>> edges = adjacencyMap[startId];
      foreach (Edge<Vertex> edge in edges) {
        if (edge.Target.Id == endId) {
          return edge;
        }
      }
      return null;
    }

    public List<Edge<Vertex>> GetInEdges(long vertexId) {
      return incidenceMap[vertexId];

    }

    public List<Edge<Vertex>> GetOutEdges(long vertexId) {
      return adjacencyMap[vertexId];
    }
  }
}
