﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.RoutePlanning.Osm {
  public static class Utils {
    public static double Distance(PointD p1, PointD p2) {
      double dx = p2.X - p1.X;
      double dy = p2.Y - p1.Y;
      return Math.Sqrt(dx * dx + dy * dy);
    }

    public static double PathLength(IList<PointD> points) {
      double length = 0;
      for (int i = 1; i < points.Count; i++) {
        length += Distance((PointD)points[i - 1], (PointD)points[i]);
      }
      return length;
    }

    public static RectangleD FindBox(IList<PointD> points) {
      double minX = double.MaxValue;
      double maxX = double.MinValue;
      double minY = double.MaxValue;
      double maxY = double.MinValue;

      foreach (PointD p in points) {
        if (p.X < minX)
          minX = p.X;
        if (p.X > maxX)
          maxX = p.X;

        if (p.Y < minY)
          minY = p.Y;
        if (p.Y > maxY)
          maxY = p.Y;
      }
      return new RectangleD(minX, minY, maxX - minX, maxY - minY);
    }

    public static double PathDistance(IList<PointD> path1, IList<PointD> path2) {
      double distance = 0;
      for (int i = 0; i < path1.Count; i++) {
        distance += Distance((PointD)path1[i], (PointD)path2[i]);
      }
      return distance / path1.Count;
    }
  }
}

